@@ -25,6 +25,7 @@
 
 #include <stdint.h>
 #include <errno.h>
+#include <stdbool.h>
 #include <string.h>
 #include <sys/socket.h>
 
@@ -53,9 +54,9 @@ struct p2p_npp_frame {
 	struct p2p_npp_ndef_entry ndefs[];
 } __attribute__((packed));
 
-static near_bool_t npp_read(int client_fd,
+static bool npp_read(int client_fd,
 			uint32_t adapter_idx, uint32_t target_idx,
-			near_tag_io_cb cb)
+			near_tag_io_cb cb, gpointer data)
 {
 	struct near_device *device;
 	struct p2p_npp_frame frame;
@@ -92,7 +93,7 @@ static near_bool_t npp_read(int client_fd,
 		DBG("NDEF %d length %d", i, ndef_length);
 
 		new_ndefs = g_try_realloc(ndefs, total_ndef_length);
-		if (new_ndefs == NULL) {
+		if (!new_ndefs) {
 			near_error("Could not allocate NDEF buffer %d",
 								bytes_recv);
 			err = -ENOMEM;
@@ -123,10 +124,10 @@ static near_bool_t npp_read(int client_fd,
 	err = near_tag_add_data(adapter_idx, target_idx,
 					ndefs, total_ndef_length);
 	if (err < 0)
-		return FALSE;
+		return false;
 
 	device = near_device_get_device(adapter_idx, target_idx);
-	if (device == NULL) {
+	if (!device) {
 		g_free(ndefs);
 		return -ENOMEM;
 	}
@@ -139,7 +140,7 @@ static near_bool_t npp_read(int client_fd,
 
 	g_free(ndefs);
 
-	return FALSE;
+	return false;
 }
 
 struct near_p2p_driver npp_driver = {
