@@ -61,10 +61,10 @@ struct near_adapter {
 	uint32_t poll_mode;
 	enum near_adapter_rf_mode rf_mode;
 
-	near_bool_t powered;
-	near_bool_t polling;
-	near_bool_t constant_poll;
-	near_bool_t dep_up;
+	bool powered;
+	bool polling;
+	bool constant_poll;
+	bool dep_up;
 
 	GHashTable *tags;
 	struct near_tag *tag_link;
@@ -79,6 +79,7 @@ struct near_adapter {
 	GList *ioreq_list;
 
 	guint presence_timeout;
+	guint dep_timer;
 };
 
 struct near_adapter_ioreq {
@@ -101,6 +102,9 @@ static void free_adapter(gpointer data)
 	if (adapter->presence_timeout > 0)
 		g_source_remove(adapter->presence_timeout);
 
+	if (adapter->dep_timer > 0)
+		g_source_remove(adapter->dep_timer);
+
 	g_free(adapter->name);
 	g_free(adapter->path);
 	g_hash_table_destroy(adapter->tags);
@@ -138,17 +142,19 @@ static char *rf_mode_to_string(struct near_adapter *adapter)
 
 static void polling_changed(struct near_adapter *adapter)
 {
+	dbus_bool_t polling;
 
+	polling = adapter->polling;
 	near_dbus_property_changed_basic(adapter->path,
 					NFC_ADAPTER_INTERFACE, "Polling",
-					DBUS_TYPE_BOOLEAN, &adapter->polling);
+					DBUS_TYPE_BOOLEAN, &polling);
 }
 
 static void rf_mode_changed(struct near_adapter *adapter)
 {
 	const char *rf_mode = rf_mode_to_string(adapter);
 
-	if (rf_mode == NULL)
+	if (!rf_mode)
 		return;
 
 	near_dbus_property_changed_basic(adapter->path,
@@ -189,7 +195,7 @@ static int adapter_start_poll(struct near_adapter *adapter)
 	if (err < 0)
 		return err;
 
-	adapter->polling = TRUE;
+	adapter->polling = true;
 
 	polling_changed(adapter);
 
@@ -203,7 +209,7 @@ static void append_path(gpointer key, gpointer value, gpointer user_data)
 
 	DBG("%s", adapter->path);
 
-	if (adapter->path == NULL)
+	if (!adapter->path)
 		return;
 
 	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH,
@@ -260,7 +266,7 @@ static void append_tag_path(gpointer key, gpointer value, gpointer user_data)
 	const char *tag_path;
 
 	tag_path = __near_tag_get_path(tag);
-	if (tag_path == NULL)
+	if (!tag_path)
 		return;
 
 	DBG("%s", tag_path);
@@ -284,7 +290,7 @@ static void append_device_path(gpointer key, gpointer value, gpointer user_data)
 	const char *device_path;
 
 	device_path = __near_device_get_path(device);
-	if (device_path == NULL)
+	if (!device_path)
 		return;
 
 	DBG("%s", device_path);
@@ -310,7 +316,7 @@ void __near_adapter_tags_changed(uint32_t adapter_idx)
 
 	adapter = g_hash_table_lookup(adapter_hash,
 					GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	near_dbus_property_changed_array(adapter->path,
@@ -327,7 +333,7 @@ void __near_adapter_devices_changed(uint32_t adapter_idx)
 
 	adapter = g_hash_table_lookup(adapter_hash,
 					GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	near_dbus_property_changed_array(adapter->path,
@@ -343,25 +349,28 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	const char *rf_mode;
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
+	dbus_bool_t val;
 
 	DBG("conn %p", conn);
 
 	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
+	if (!reply)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &array);
 
 	near_dbus_dict_open(&array, &dict);
 
+	val = adapter->powered;
 	near_dbus_dict_append_basic(&dict, "Powered",
-				    DBUS_TYPE_BOOLEAN, &adapter->powered);
+					DBUS_TYPE_BOOLEAN, &val);
 
+	val = adapter->polling;
 	near_dbus_dict_append_basic(&dict, "Polling",
-				    DBUS_TYPE_BOOLEAN, &adapter->polling);
+				    DBUS_TYPE_BOOLEAN, &val);
 
 	rf_mode = rf_mode_to_string(adapter);
-	if (rf_mode != NULL)
+	if (rf_mode)
 		near_dbus_dict_append_basic(&dict, "Mode",
 						DBUS_TYPE_STRING, &rf_mode);
 
@@ -389,7 +398,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
-	if (dbus_message_iter_init(msg, &iter) == FALSE)
+	if (!dbus_message_iter_init(msg, &iter))
 		return __near_error_invalid_arguments(msg);
 
 	dbus_message_iter_get_basic(&iter, &name);
@@ -398,8 +407,8 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 	type = dbus_message_iter_get_arg_type(&value);
 
-	if (g_str_equal(name, "Powered") == TRUE) {
-		near_bool_t powered;
+	if (g_str_equal(name, "Powered")) {
+		dbus_bool_t powered;
 
 		if (type != DBUS_TYPE_BOOLEAN)
 			return __near_error_invalid_arguments(msg);
@@ -409,7 +418,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 		err = __near_netlink_adapter_enable(adapter->idx, powered);
 		if (err < 0) {
 			if (err == -EALREADY) {
-				if (powered == TRUE)
+				if (powered)
 					return __near_error_already_enabled(msg);
 				else
 					return __near_error_already_disabled(msg);
@@ -464,14 +473,14 @@ static DBusMessage *stop_poll_loop(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
-	if (adapter->polling == FALSE)
+	if (!adapter->polling)
 		return __near_error_not_polling(msg);
 
 	err = __near_netlink_stop_poll(adapter->idx);
 	if (err < 0)
 		return __near_error_failed(msg, -err);
 
-	adapter->polling = FALSE;
+	adapter->polling = false;
 
 	polling_changed(adapter);
 
@@ -489,11 +498,11 @@ static gboolean check_presence(gpointer user_data)
 
 	DBG("");
 
-	if (adapter == NULL)
+	if (!adapter)
 		return FALSE;
 
 	tag = adapter->tag_link;
-	if (tag == NULL)
+	if (!tag)
 		goto out_err;
 
 	err = __near_tag_check_presence(tag, tag_present_cb);
@@ -506,12 +515,26 @@ static gboolean check_presence(gpointer user_data)
 
 out_err:
 	near_adapter_disconnect(adapter->idx);
-	if (adapter->constant_poll == TRUE)
+	if (adapter->constant_poll)
 		adapter_start_poll(adapter);
 
 	return FALSE;
 }
 
+static gboolean dep_timer(gpointer user_data)
+{
+	struct near_adapter *adapter = user_data;
+
+	DBG("");
+
+	if (!adapter)
+		return FALSE;
+
+	adapter_start_poll(adapter);
+
+	return FALSE;
+}
+
 static void tag_present_cb(uint32_t adapter_idx, uint32_t target_idx,
 								int status)
 {
@@ -521,14 +544,14 @@ static void tag_present_cb(uint32_t adapter_idx, uint32_t target_idx,
 
 	adapter = g_hash_table_lookup(adapter_hash,
 					GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	if (status < 0) {
 		DBG("Tag is gone");
 
 		near_adapter_disconnect(adapter->idx);
-		if (adapter->constant_poll == TRUE)
+		if (adapter->constant_poll)
 			adapter_start_poll(adapter);
 
 		return;
@@ -548,7 +571,7 @@ void __near_adapter_start_check_presence(uint32_t adapter_idx,
 
 	adapter = g_hash_table_lookup(adapter_hash,
 			GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	adapter->presence_timeout =
@@ -565,7 +588,7 @@ void __near_adapter_stop_check_presence(uint32_t adapter_idx,
 
 	adapter = g_hash_table_lookup(adapter_hash,
 			GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	if (adapter->presence_timeout > 0)
@@ -594,25 +617,25 @@ static const GDBusSignalTable adapter_signals[] = {
 };
 
 struct near_adapter *__near_adapter_create(uint32_t idx,
-		const char *name, uint32_t protocols, near_bool_t powered)
+		const char *name, uint32_t protocols, bool powered)
 {
 	struct near_adapter *adapter;
-	near_bool_t powered_setting;
+	bool powered_setting;
 
 	adapter = g_try_malloc0(sizeof(struct near_adapter));
-	if (adapter == NULL)
+	if (!adapter)
 		return NULL;
 
 	adapter->name = g_strdup(name);
-	if (adapter->name == NULL) {
+	if (!adapter->name) {
 		g_free(adapter);
 		return NULL;
 	}
 
 	powered_setting = near_setting_get_bool("DefaultPowered");
-	if (powered_setting == TRUE && powered == FALSE &&
+	if (powered_setting && !powered &&
 	    !__near_netlink_adapter_enable(idx, powered_setting))
-			powered = TRUE;
+			powered = true;
 
 	DBG("Powered %d", powered);
 
@@ -620,7 +643,7 @@ struct near_adapter *__near_adapter_create(uint32_t idx,
 	adapter->protocols = protocols;
 	adapter->powered = powered;
 	adapter->constant_poll = near_setting_get_bool("ConstantPoll");
-	adapter->dep_up = FALSE;
+	adapter->dep_up = false;
 	adapter->tags = g_hash_table_new_full(g_direct_hash, g_direct_equal,
 							NULL, free_tag);
 	adapter->tag_sock = -1;
@@ -651,42 +674,45 @@ struct near_adapter *__near_adapter_get(uint32_t idx)
 	return g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
 }
 
-int __near_adapter_set_dep_state(uint32_t idx, near_bool_t dep)
+int __near_adapter_set_dep_state(uint32_t idx, bool dep)
 {
 	struct near_adapter *adapter;
 
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
 	adapter->dep_up = dep;
 
-	if (dep == FALSE && adapter->constant_poll == TRUE)
+	if (!dep && adapter->constant_poll)
 		adapter_start_poll(adapter);
 
-	if (dep == FALSE) {
+	if (!dep) {
 		uint32_t target_idx;
 
 		target_idx =  __neard_device_get_idx(adapter->device_link);
 		__near_adapter_remove_target(idx, target_idx);
 	} else {
+		if (adapter->dep_timer > 0)
+			g_source_remove(adapter->dep_timer);
+
 		__near_adapter_devices_changed(idx);
 	}
 
 	return 0;
 }
 
-near_bool_t __near_adapter_get_dep_state(uint32_t idx)
+bool __near_adapter_get_dep_state(uint32_t idx)
 {
 	struct near_adapter *adapter;
 
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
-		return FALSE;
+	if (!adapter)
+		return false;
 
 	return adapter->dep_up;
 }
@@ -697,7 +723,7 @@ int __near_adapter_add(struct near_adapter *adapter)
 
 	DBG("%s", adapter->path);
 
-	if (g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx)) != NULL)
+	if (g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx)))
 		return -EEXIST;
 
 	g_hash_table_insert(adapter_hash, GINT_TO_POINTER(idx), adapter);
@@ -730,12 +756,12 @@ static void tag_read_cb(uint32_t adapter_idx, uint32_t target_idx, int status)
 
 	adapter = g_hash_table_lookup(adapter_hash,
 					GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	if (status < 0) {
 		near_adapter_disconnect(adapter->idx);
-		if (adapter->constant_poll == TRUE)
+		if (adapter->constant_poll)
 			adapter_start_poll(adapter);
 
 		return;
@@ -757,16 +783,16 @@ static void device_read_cb(uint32_t adapter_idx, uint32_t target_idx,
 
 	adapter = g_hash_table_lookup(adapter_hash,
 					GINT_TO_POINTER(adapter_idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return;
 
 	if (status < 0) {
-		if (adapter->device_link != NULL) {
+		if (adapter->device_link) {
 			__near_netlink_dep_link_down(adapter->idx);
 			adapter->device_link = NULL;
 		}
 
-		if (adapter->constant_poll == TRUE)
+		if (adapter->constant_poll)
 			adapter_start_poll(adapter);
 
 		return;
@@ -785,7 +811,7 @@ static int adapter_add_tag(struct near_adapter *adapter, uint32_t target_idx,
 	tag = __near_tag_add(adapter->idx, target_idx, protocols,
 				sens_res, sel_res,
 				nfcid, nfcid_len);
-	if (tag == NULL)
+	if (!tag)
 		return -ENODEV;
 
 	g_hash_table_insert(adapter->tags, GINT_TO_POINTER(target_idx), tag);
@@ -817,7 +843,7 @@ static int adapter_add_device(struct near_adapter *adapter,
 	int err;
 
 	device = __near_device_add(adapter->idx, target_idx, nfcid, nfcid_len);
-	if (device == NULL)
+	if (!device)
 		return -ENODEV;
 
 	g_hash_table_insert(adapter->devices, GINT_TO_POINTER(target_idx),
@@ -832,7 +858,7 @@ static int adapter_add_device(struct near_adapter *adapter,
 
 	adapter->device_link = device;
 
-	if (adapter->dep_up == TRUE)
+	if (adapter->dep_up)
 		return 0;
 
 	err = __near_netlink_dep_link_up(adapter->idx, target_idx,
@@ -841,6 +867,10 @@ static int adapter_add_device(struct near_adapter *adapter,
 	if (err < 0)
 		adapter->device_link = NULL;
 
+	DBG("Starting DEP timer");
+
+	adapter->dep_timer = g_timeout_add_seconds(1, dep_timer, adapter);
+
 	return err;
 }
 
@@ -849,25 +879,31 @@ int __near_adapter_add_target(uint32_t idx, uint32_t target_idx,
 			uint8_t *nfcid, uint8_t nfcid_len)
 {
 	struct near_adapter *adapter;
+	int ret;
 
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
-	adapter->polling = FALSE;
+	adapter->polling = false;
 	polling_changed(adapter);
 
 	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_INITIATOR;
 	rf_mode_changed(adapter);
 
 	if (protocols & NFC_PROTO_NFC_DEP_MASK)
-		return adapter_add_device(adapter, target_idx,
+		ret = adapter_add_device(adapter, target_idx,
 						nfcid, nfcid_len);
 	else
-		return adapter_add_tag(adapter, target_idx, protocols,
+		ret = adapter_add_tag(adapter, target_idx, protocols,
 					sens_res, sel_res, nfcid, nfcid_len);
+
+	if (ret < 0 && adapter->constant_poll)
+		adapter_start_poll(adapter);
+
+	return ret;
 }
 
 int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx)
@@ -877,21 +913,21 @@ int __near_adapter_remove_target(uint32_t idx, uint32_t target_idx)
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
 	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_IDLE;
 	rf_mode_changed(adapter);
 
 	if (g_hash_table_remove(adapter->tags,
-			GINT_TO_POINTER(target_idx)) == TRUE) {
+			GINT_TO_POINTER(target_idx))) {
 		__near_adapter_tags_changed(idx);
 
 		return 0;
 	}
 
 	if (g_hash_table_remove(adapter->devices,
-			GINT_TO_POINTER(target_idx)) == TRUE) {
+			GINT_TO_POINTER(target_idx))) {
 		__near_adapter_devices_changed(idx);
 
 		return 0;
@@ -908,11 +944,11 @@ int __near_adapter_add_device(uint32_t idx, uint8_t *nfcid, uint8_t nfcid_len)
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
-	adapter->polling = FALSE;
-	adapter->dep_up = TRUE;
+	adapter->polling = false;
+	adapter->dep_up = true;
 	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_TARGET;
 	polling_changed(adapter);
 	rf_mode_changed(adapter);
@@ -934,20 +970,19 @@ int __near_adapter_remove_device(uint32_t idx)
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
-	if (g_hash_table_remove(adapter->devices,
-			GINT_TO_POINTER(device_idx)) == FALSE)
+	if (!g_hash_table_remove(adapter->devices, GINT_TO_POINTER(device_idx)))
 		return 0;
 
 	adapter->rf_mode = NEAR_ADAPTER_RF_MODE_IDLE;
 	rf_mode_changed(adapter);
 	__near_adapter_devices_changed(idx);
 
-	adapter->dep_up = FALSE;
+	adapter->dep_up = false;
 
-	if (adapter->constant_poll == TRUE)
+	if (adapter->constant_poll)
 		adapter_start_poll(adapter);
 
 	return 0;
@@ -960,7 +995,7 @@ static void adapter_flush_rx(struct near_adapter *adapter, int error)
 	for (list = adapter->ioreq_list; list; list = list->next) {
 		struct near_adapter_ioreq *req = list->data;
 
-		if (req == NULL)
+		if (!req)
 			continue;
 
 		req->cb(NULL, error, req->data);
@@ -1009,7 +1044,7 @@ static gboolean adapter_recv_event(GIOChannel *channel, GIOCondition condition,
 
 	sk = g_io_channel_unix_get_fd(channel);
 	first = g_list_first(adapter->ioreq_list);
-	if (first == NULL)
+	if (!first)
 		return TRUE;
 
 	req = first->data;
@@ -1032,7 +1067,7 @@ int near_adapter_connect(uint32_t idx, uint32_t target_idx, uint8_t protocol)
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
 	if (adapter->tag_sock != -1)
@@ -1040,7 +1075,7 @@ int near_adapter_connect(uint32_t idx, uint32_t target_idx, uint8_t protocol)
 
 	tag = g_hash_table_lookup(adapter->tags,
 				GINT_TO_POINTER(target_idx));
-	if (tag == NULL)
+	if (!tag)
 		return -ENOLINK;
 
 	sock = socket(AF_NFC, SOCK_SEQPACKET, NFC_SOCKPROTO_RAW);
@@ -1061,7 +1096,7 @@ int near_adapter_connect(uint32_t idx, uint32_t target_idx, uint8_t protocol)
 	adapter->tag_sock = sock;
 	adapter->tag_link = tag;
 
-	if (adapter->channel == NULL)
+	if (!adapter->channel)
 		adapter->channel = g_io_channel_unix_new(adapter->tag_sock);
 
 	g_io_channel_set_flags(adapter->channel, G_IO_FLAG_NONBLOCK, NULL);
@@ -1084,12 +1119,12 @@ int near_adapter_disconnect(uint32_t idx)
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL)
+	if (!adapter)
 		return -ENODEV;
 
 	DBG("link %p", adapter->tag_link);
 
-	if (adapter->tag_link == NULL)
+	if (!adapter->tag_link)
 		return -ENOLINK;
 
 	tag_type = __near_tag_get_type(adapter->tag_link);
@@ -1125,19 +1160,19 @@ int near_adapter_send(uint32_t idx, uint8_t *buf, size_t length,
 	DBG("idx %d", idx);
 
 	adapter = g_hash_table_lookup(adapter_hash, GINT_TO_POINTER(idx));
-	if (adapter == NULL) {
+	if (!adapter) {
 		err = -ENODEV;
 		goto out_err;
 	}
 
-	if (adapter->tag_sock == -1 || adapter->tag_link == NULL) {
+	if (adapter->tag_sock == -1 || !adapter->tag_link) {
 		err = -ENOLINK;
 		goto out_err;
 	}
 
-	if (cb != NULL && adapter->watch != 0) {
+	if (cb && adapter->watch != 0) {
 		req = g_try_malloc0(sizeof(*req));
-		if (req == NULL) {
+		if (!req) {
 			err = -ENOMEM;
 			goto out_err;
 		}
@@ -1159,7 +1194,7 @@ int near_adapter_send(uint32_t idx, uint8_t *buf, size_t length,
 	return err;
 
 out_err:
-	if (req != NULL) {
+	if (req) {
 		GList *last = g_list_last(adapter->ioreq_list);
 
 		g_free(req);
@@ -1167,7 +1202,7 @@ out_err:
 				g_list_delete_link(adapter->ioreq_list, last);
 	}
 
-	if (data_rel != NULL)
+	if (data_rel)
 		return (*data_rel)(err, data);
 
 	return err;
@@ -1180,7 +1215,7 @@ static void adapter_listen(gpointer key, gpointer value, gpointer user_data)
 
 	DBG("%s", adapter->path);
 
-	if (adapter->path == NULL)
+	if (!adapter->path)
 		return;
 
 	driver->listen(adapter->idx, device_read_cb);
